@@ -672,3 +672,33 @@ class ProfileServer(DashboardComponent):
         times = [t * 1000 for t, _ in self.log]
         counts = list(toolz.pluck('count', toolz.pluck(1, self.log)))
         self.ts_source.data.update({'time': times, 'count': counts})
+
+
+def add_periodic_callback(doc, component, interval):
+    """ Add periodic callback to doc in a way that avoids reference cycles
+
+    If we instead use ``doc.add_periodic_callback(component.update, 100)`` then
+    the component stays in memory as a reference cycle because its method is
+    still around.  This way we avoid that and let things clean up a bit more
+    nicely.
+
+    TODO: we still have reference cycles.  Docs seem to be referred to by their
+    add_periodic_callback methods.
+    """
+    ref = weakref.ref(component)
+
+    doc.add_periodic_callback(lambda: update(ref), interval)
+    _attach(doc, component)
+
+
+def update(ref):
+    comp = ref()
+    if comp is not None:
+        comp.update()
+
+
+def _attach(doc, component):
+    if not hasattr(doc, 'components'):
+        doc.components = set()
+
+    doc.components.add(component)
