@@ -7,13 +7,14 @@ import os
 import psutil
 import shutil
 import threading
+import uuid
 
+import dask
 from tornado import gen
 from tornado.ioloop import IOLoop, TimeoutError
 from tornado.locks import Event
 
 from .comm import get_address_host, get_local_address_for, unparse_host_port
-from .config import config
 from .core import rpc, RPCClosed, CommClosedError, coerce_to_address
 from .metrics import time
 from .node import ServerNode
@@ -22,8 +23,7 @@ from .proctitle import enable_proctitle_on_children
 from .security import Security
 from .utils import (get_ip, mp_context, silence_logging, json_load_robust,
         PeriodicCallback)
-from .worker import _ncores, run, parse_memory_limit
-
+from .worker import _ncores, run, parse_memory_limit, Worker
 
 logger = logging.getLogger(__name__)
 
@@ -43,12 +43,13 @@ class Nanny(ServerNode):
                  name=None, memory_limit='auto', reconnect=True,
                  validate=False, quiet=False, resources=None, silence_logs=None,
                  death_timeout=None, preload=(), preload_argv=[], security=None,
-                 contact_address=None, listen_address=None, **kwargs):
+                 contact_address=None, listen_address=None, worker_class=None,
+                 **kwargs):
         if scheduler_file:
             cfg = json_load_robust(scheduler_file)
             self.scheduler_addr = cfg['address']
-        elif scheduler_ip is None and config.get('scheduler-address'):
-            self.scheduler_addr = config['scheduler-address']
+        elif scheduler_ip is None and dask.config.get('scheduler-address'):
+            self.scheduler_addr = dask.config.get('scheduler-address')
         elif scheduler_port is None:
             self.scheduler_addr = coerce_to_address(scheduler_ip)
         else:
@@ -61,9 +62,10 @@ class Nanny(ServerNode):
         self.death_timeout = death_timeout
         self.preload = preload
         self.preload_argv = preload_argv
+        self.Worker = Worker if worker_class is None else worker_class
 
         self.contact_address = contact_address
-        self.memory_terminate_fraction = config.get('worker-memory-terminate', 0.95)
+        self.memory_terminate_fraction = dask.config.get('distributed.worker.memory.terminate')
 
         self.security = security or Security()
         assert isinstance(self.security, Security)
@@ -213,6 +215,7 @@ class Nanny(ServerNode):
                 worker_start_args=(start_arg,),
                 silence_logs=self.silence_logs,
                 on_exit=self._on_exit,
+                worker=self.Worker
             )
 
         self.auto_restart = True
@@ -248,14 +251,22 @@ class Nanny(ServerNode):
             raise gen.Return('OK')
 
     def memory_monitor(self):
-        """ Track worker's memory.  Restart if it goes above 95% """
+        """ Track worker's memory.  Restart if it goes above terminate fraction """
         if self.status != 'running':
             return
-        memory = psutil.Process(self.process.pid).memory_info().rss
+        process = self.process.process
+        if process is None:
+            return
+        try:
+            proc = psutil.Process(process.pid)
+        except psutil.NoSuchProcess:
+            return
+        memory = proc.memory_info().rss
         frac = memory / self.memory_limit
         if self.memory_terminate_fraction and frac > self.memory_terminate_fraction:
-            logger.warning("Worker exceeded 95% memory budget.  Restarting")
-            self.process.process.terminate()
+            logger.warning("Worker exceeded %d%% memory budget. Restarting",
+                           100 * self.memory_terminate_fraction)
+            process.terminate()
 
     def is_alive(self):
         return self.process is not None and self.process.status == 'running'
@@ -311,7 +322,7 @@ class Nanny(ServerNode):
 class WorkerProcess(object):
 
     def __init__(self, worker_args, worker_kwargs, worker_start_args,
-                 silence_logs, on_exit):
+                 silence_logs, on_exit, worker):
         self.status = 'init'
         self.silence_logs = silence_logs
         self.worker_args = worker_args
@@ -319,6 +330,7 @@ class WorkerProcess(object):
         self.worker_start_args = worker_start_args
         self.on_exit = on_exit
         self.process = None
+        self.Worker = worker
 
         # Initialized when worker is ready
         self.worker_dir = None
@@ -336,47 +348,36 @@ class WorkerProcess(object):
             yield self.running.wait()
             raise gen.Return(self.status)
 
-        while True:
-            # FIXME: this sometimes stalls in _wait_until_connected
-            # our temporary solution is to retry a few times if the process
-            # doesn't start up in five seconds
-            self.init_result_q = mp_context.Queue()
-            self.child_stop_q = mp_context.Queue()
-            try:
-                self.process = AsyncProcess(
-                    target=self._run,
-                    kwargs=dict(worker_args=self.worker_args,
-                                worker_kwargs=self.worker_kwargs,
-                                worker_start_args=self.worker_start_args,
-                                silence_logs=self.silence_logs,
-                                init_result_q=self.init_result_q,
-                                child_stop_q=self.child_stop_q),
-                )
-                self.process.daemon = True
-                self.process.set_exit_callback(self._on_exit)
-                self.running = Event()
-                self.stopped = Event()
-                self.status = 'starting'
-                yield self.process.start()
-                if self.status == 'starting':
-                    yield gen.with_timeout(timedelta(seconds=5),
-                                           self._wait_until_started())
-            except gen.TimeoutError:
-                logger.info("Failed to start worker process.  Restarting")
-                yield gen.with_timeout(timedelta(seconds=1),
-                                       self.process.terminate())
-            else:
-                break
+        self.init_result_q = init_q = mp_context.Queue()
+        self.child_stop_q = mp_context.Queue()
+        uid = uuid.uuid4().hex
 
-        if self.status == 'starting':
-            msg = yield self._wait_until_connected()
-            if not msg:
-                raise gen.Return(self.status)
-            self.worker_address = msg['address']
-            self.worker_dir = msg['dir']
-            assert self.worker_address
-            self.status = 'running'
-            self.running.set()
+        self.process = AsyncProcess(
+            target=self._run,
+            kwargs=dict(worker_args=self.worker_args,
+                        worker_kwargs=self.worker_kwargs,
+                        worker_start_args=self.worker_start_args,
+                        silence_logs=self.silence_logs,
+                        init_result_q=self.init_result_q,
+                        child_stop_q=self.child_stop_q,
+                        uid=uid, Worker=self.Worker),
+        )
+        self.process.daemon = True
+        self.process.set_exit_callback(self._on_exit)
+        self.running = Event()
+        self.stopped = Event()
+        self.status = 'starting'
+        yield self.process.start()
+        msg = yield self._wait_until_connected(uid)
+        if not msg:
+            raise gen.Return(self.status)
+        self.worker_address = msg['address']
+        self.worker_dir = msg['dir']
+        assert self.worker_address
+        self.status = 'running'
+        self.running.set()
+
+        init_q.close()
 
         raise gen.Return(self.status)
 
@@ -444,10 +445,12 @@ class WorkerProcess(object):
         self.status = 'stopping'
 
         process = self.process
-        self.child_stop_q.put({'op': 'stop',
-                               'timeout': max(0, deadline - loop.time()) * 0.8,
-                               'executor_wait': executor_wait,
-                               })
+        self.child_stop_q.put({
+            'op': 'stop',
+            'timeout': max(0, deadline - loop.time()) * 0.8,
+            'executor_wait': executor_wait,
+        })
+        self.child_stop_q.close()
 
         while process.is_alive() and loop.time() < deadline:
             yield gen.sleep(0.05)
@@ -461,21 +464,7 @@ class WorkerProcess(object):
                 logger.error("Failed to kill worker process: %s", e)
 
     @gen.coroutine
-    def _wait_until_started(self):
-        delay = 0.05
-        while True:
-            if self.status != 'starting':
-                return
-            try:
-                msg = self.init_result_q.get_nowait()
-                assert msg == 'started', msg
-                return
-            except Empty:
-                yield gen.sleep(delay)
-                continue
-
-    @gen.coroutine
-    def _wait_until_connected(self):
+    def _wait_until_connected(self, uid):
         delay = 0.05
         while True:
             if self.status != 'starting':
@@ -486,9 +475,12 @@ class WorkerProcess(object):
                 yield gen.sleep(delay)
                 continue
 
-            if isinstance(msg, Exception):
-                logger.error("Failed while trying to start worker process",
-                             exc_info=True)
+            if msg['uid'] != uid:  # ensure that we didn't cross queues
+                continue
+
+            if 'exception' in msg:
+                logger.error("Failed while trying to start worker process: %s",
+                             msg['exception'])
                 yield self.process.join()
                 raise msg
             else:
@@ -496,9 +488,7 @@ class WorkerProcess(object):
 
     @classmethod
     def _run(cls, worker_args, worker_kwargs, worker_start_args,
-             silence_logs, init_result_q, child_stop_q):  # pragma: no cover
-        from distributed import Worker
-
+             silence_logs, init_result_q, child_stop_q, uid, Worker):  # pragma: no cover
         try:
             from dask.multiprocessing import initialize_worker_process
         except ImportError:   # old Dask version
@@ -535,6 +525,7 @@ class WorkerProcess(object):
                 except Empty:
                     pass
                 else:
+                    child_stop_q.close()
                     assert msg.pop('op') == 'stop'
                     loop.add_callback(do_stop, **msg)
                     break
@@ -548,16 +539,18 @@ class WorkerProcess(object):
             """
             Try to start worker and inform parent of outcome.
             """
-            init_result_q.put('started')
             try:
                 yield worker._start(*worker_start_args)
             except Exception as e:
                 logger.exception("Failed to start worker")
-                init_result_q.put(e)
+                init_result_q.put({'uid': uid, 'exception': e})
+                init_result_q.close()
             else:
                 assert worker.address
                 init_result_q.put({'address': worker.address,
-                                   'dir': worker.local_dir})
+                                   'dir': worker.local_dir,
+                                   'uid': uid})
+                init_result_q.close()
                 yield worker.wait_until_closed()
                 logger.info("Worker closed")
 
